@@ -54,6 +54,11 @@ class NumericalTransformer(BaseTransformer):
             is given, there won't be a maximum.
     """
 
+    INPUT_TYPE = 'numerical'
+    DETERMINISTIC_TRANSFORM = True
+    DETERMINISTIC_REVERSE = True
+    COMPOSITION_IS_IDENTITY = True
+
     null_transformer = None
     nan = None
     _dtype = None
@@ -70,11 +75,39 @@ class NumericalTransformer(BaseTransformer):
         self.min_value = min_value
         self.max_value = max_value
 
+    def get_output_types(self):
+        """Return the output types supported by the transformer.
+
+        Returns:
+            dict:
+                Mapping from the transformed column names to supported data types.
+        """
+        output_types = {
+            'value': 'float',
+        }
+        if self.null_transformer and self.null_transformer.creates_null_column():
+            output_types['is_null'] = 'float'
+
+        return self._add_prefix(output_types)
+
+    def is_composition_identity(self):
+        """Return whether composition of transform and reverse transform produces the input data.
+
+        Returns:
+            bool:
+                Whether or not transforming and then reverse transforming returns the input data.
+        """
+        if self.null_transformer and not self.null_transformer.creates_null_column():
+            return False
+
+        return self.COMPOSITION_IS_IDENTITY
+
     @staticmethod
     def _learn_rounding_digits(data):
         # check if data has any decimals
-        roundable_data = data[~(np.isinf(data) | pd.isnull(data))]
-        if (roundable_data % 1 != 0).any():
+        data = np.array(data)
+        roundable_data = data[~(np.isinf(data) | pd.isna(data))]
+        if ((roundable_data % 1) != 0).any():
             if not (roundable_data == roundable_data.round(MAX_DECIMALS)).all():
                 return None
 
@@ -91,16 +124,13 @@ class NumericalTransformer(BaseTransformer):
 
         return None
 
-    def fit(self, data):
+    def _fit(self, data):
         """Fit the transformer to the data.
 
         Args:
-            data (pandas.Series or numpy.ndarray):
+            data (pandas.DataFrame or pandas.Series):
                 Data to fit.
         """
-        if isinstance(data, np.ndarray):
-            data = pd.Series(data)
-
         self._dtype = self.dtype or data.dtype
         self._min_value = data.min() if self.min_value == 'auto' else self.min_value
         self._max_value = data.max() if self.max_value == 'auto' else self.max_value
@@ -113,34 +143,34 @@ class NumericalTransformer(BaseTransformer):
         self.null_transformer = NullTransformer(self.nan, self.null_column, copy=True)
         self.null_transformer.fit(data)
 
-    def transform(self, data):
+    def _transform(self, data):
         """Transform numerical data.
 
         Integer values are replaced by their float equivalent. Non null float values
         are left unmodified.
 
         Args:
-            data (pandas.Series or numpy.ndarray):
+            data (pandas.Series):
                 Data to transform.
 
         Returns:
             numpy.ndarray
         """
-        if isinstance(data, np.ndarray):
-            data = pd.Series(data)
-
         return self.null_transformer.transform(data)
 
-    def reverse_transform(self, data):
+    def _reverse_transform(self, data):
         """Convert data back into the original format.
 
         Args:
-            data (numpy.ndarray):
+            data (pd.Series or numpy.ndarray):
                 Data to transform.
 
         Returns:
             numpy.ndarray
         """
+        if not isinstance(data, np.ndarray):
+            data = data.to_numpy()
+
         if self._min_value is not None or self._max_value is not None:
             if len(data.shape) > 1:
                 data[:, 0] = data[:, 0].clip(self._min_value, self._max_value)
@@ -154,12 +184,115 @@ class NumericalTransformer(BaseTransformer):
         if self._rounding_digits is not None or is_integer:
             data = data.round(self._rounding_digits or 0)
 
-        if pd.isnull(data).any() and is_integer:
+        if pd.isna(data).any() and is_integer:
             return data
 
         return data.astype(self._dtype)
 
 
+class NumericalRoundedBoundedTransformer(NumericalTransformer):
+    """Transformer for numerical data.
+
+    This transformer replaces integer values with their float equivalent, bounded by the fitted
+    data (the minimum and maximum values seen while fitting). It will also round all values to
+    the maximum number of decimal places detected in the fitted data.
+
+    Non null float values are not modified.
+
+    This class behaves exactly as the ``NumericalTransformer`` with ``min_value='auto'``,
+    ``max_value='auto'`` and ``rounding='auto'``.
+
+    Args:
+        dtype (data type):
+            Data type of the data to transform. It will be used when reversing the
+            transformation. If not provided, the dtype of the fit data will be used.
+            Defaults to ``None``.
+        nan (int, str or None):
+            Indicate what to do with the null values. If an integer is given, replace them
+            with the given value. If the strings ``'mean'`` or ``'mode'`` are given, replace
+            them with the corresponding aggregation. If ``None`` is given, do not replace them.
+            Defaults to ``'mean'``.
+        null_column (bool):
+            Whether to create a new column to indicate which values were null or not.
+            If ``None``, only create a new column when the data contains null values.
+            If ``True``, always create the new column whether there are null values or not.
+            If ``False``, do not create the new column.
+            Defaults to ``None``.
+    """
+
+    def __init__(self, dtype=None, nan='mean', null_column=None):
+        super().__init__(dtype=dtype, nan=nan, null_column=null_column, min_value='auto',
+                         max_value='auto', rounding='auto')
+
+
+class NumericalBoundedTransformer(NumericalTransformer):
+    """Transformer for numerical data.
+
+    This transformer replaces integer values with their float equivalent, bounded by the fitted
+    data (the minimum and maximum values seen while fitting).
+
+    Non null float values are not modified.
+
+    This class behaves exactly as the ``NumericalTransformer`` with ``min_value='auto'``,
+    ``max_value='auto'`` and ``rounding=None``.
+
+    Args:
+        dtype (data type):
+            Data type of the data to transform. It will be used when reversing the
+            transformation. If not provided, the dtype of the fit data will be used.
+            Defaults to ``None``.
+        nan (int, str or None):
+            Indicate what to do with the null values. If an integer is given, replace them
+            with the given value. If the strings ``'mean'`` or ``'mode'`` are given, replace
+            them with the corresponding aggregation. If ``None`` is given, do not replace them.
+            Defaults to ``'mean'``.
+        null_column (bool):
+            Whether to create a new column to indicate which values were null or not.
+            If ``None``, only create a new column when the data contains null values.
+            If ``True``, always create the new column whether there are null values or not.
+            If ``False``, do not create the new column.
+            Defaults to ``None``.
+    """
+
+    def __init__(self, dtype=None, nan='mean', null_column=None):
+        super().__init__(dtype=dtype, nan=nan, null_column=null_column, min_value='auto',
+                         max_value='auto', rounding=None)
+
+
+class NumericalRoundedTransformer(NumericalTransformer):
+    """Transformer for numerical data.
+
+    This transformer replaces integer values with their float equivalent, rounding all values to
+    the maximum number of decimal places detected in the fitted data.
+
+    Non null float values are not modified.
+
+    This class behaves exactly as the ``NumericalTransformer`` with ``min_value=None``,
+    ``max_value=None`` and ``rounding='auto'``.
+
+    Args:
+        dtype (data type):
+            Data type of the data to transform. It will be used when reversing the
+            transformation. If not provided, the dtype of the fit data will be used.
+            Defaults to ``None``.
+        nan (int, str or None):
+            Indicate what to do with the null values. If an integer is given, replace them
+            with the given value. If the strings ``'mean'`` or ``'mode'`` are given, replace
+            them with the corresponding aggregation. If ``None`` is given, do not replace them.
+            Defaults to ``'mean'``.
+        null_column (bool):
+            Whether to create a new column to indicate which values were null or not.
+            If ``None``, only create a new column when the data contains null values.
+            If ``True``, always create the new column whether there are null values or not.
+            If ``False``, do not create the new column.
+            Defaults to ``None``.
+    """
+
+    def __init__(self, dtype=None, nan='mean', null_column=None):
+        super().__init__(dtype=dtype, nan=nan, null_column=null_column, min_value=None,
+                         max_value=None, rounding='auto')
+
+
 class GaussianCopulaTransformer(NumericalTransformer):
     r"""Transformer for numerical data based on copulas transformation.
 
@@ -222,6 +355,7 @@ class GaussianCopulaTransformer(NumericalTransformer):
     """
 
     _univariate = None
+    COMPOSITION_IS_IDENTITY = False
 
     def __init__(self, dtype=None, nan='mean', null_column=None, distribution='parametric'):
         super().__init__(dtype=dtype, nan=nan, null_column=null_column)
@@ -294,19 +428,19 @@ class GaussianCopulaTransformer(NumericalTransformer):
            issubclass(distribution, self._distributions['univariate']):
             return distribution()
 
-        raise TypeError('Invalid distribution: {}'.format(distribution))
+        raise TypeError(f'Invalid distribution: {distribution}')
 
-    def fit(self, data):
+    def _fit(self, data):
         """Fit the transformer to the data.
 
         Args:
-            data (pandas.Series or numpy.ndarray):
+            data (pandas.Series):
                 Data to fit to.
         """
         self._univariate = self._get_univariate()
 
-        super().fit(data)
-        data = super().transform(data)
+        super()._fit(data)
+        data = super()._transform(data)
         if data.ndim > 1:
             data = data[:, 0]
 
@@ -316,17 +450,17 @@ class GaussianCopulaTransformer(NumericalTransformer):
         cdf = self._univariate.cdf(data)
         return scipy.stats.norm.ppf(cdf.clip(0 + EPSILON, 1 - EPSILON))
 
-    def transform(self, data):
+    def _transform(self, data):
         """Transform numerical data.
 
         Args:
-            data (pandas.Series or numpy.ndarray):
+            data (pandas.Series):
                 Data to transform.
 
         Returns:
             numpy.ndarray
         """
-        transformed = super().transform(data)
+        transformed = super()._transform(data)
         if transformed.ndim > 1:
             transformed[:, 0] = self._copula_transform(transformed[:, 0])
         else:
@@ -334,19 +468,22 @@ class GaussianCopulaTransformer(NumericalTransformer):
 
         return transformed
 
-    def reverse_transform(self, data):
+    def _reverse_transform(self, data):
         """Convert data back into the original format.
 
         Args:
-            data (numpy.ndarray):
+            data (pd.Series or numpy.ndarray):
                 Data to transform.
 
         Returns:
             pandas.Series
         """
+        if not isinstance(data, np.ndarray):
+            data = data.to_numpy()
+
         if data.ndim > 1:
             data[:, 0] = self._univariate.ppf(scipy.stats.norm.cdf(data[:, 0]))
         else:
             data = self._univariate.ppf(scipy.stats.norm.cdf(data))
 
-        return super().reverse_transform(data)
+        return super()._reverse_transform(data)
